pub struct BasicHeader {
pub version: u8,
pub next_header: NextAfterBasic,
pub reserved: Bits<8>,
pub lifetime: Lifetime,
pub remaining_hop_limit: u8,
}Fields
version: u8Identifies the version of the GeoNetworking protocol
next_header: NextAfterBasicIdentifies the type of header immediately following the GeoNetworking Basic Header
reserved: Bits<8>Reserved. Set to 0
lifetime: LifetimeLifetime field. Indicates the maximum tolerable time a packet may be buffered until it reaches its destination Bit 0 to Bit 5: LT sub-field Multiplier Bit 6 to Bit 7: LT sub-field Base
remaining_hop_limit: u8Decremented by 1 by each GeoAdhoc router that forwards the packet The packet shall not be forwarded if RHL is decremented to zero